from setuptools import setup

setup(
    name='Sudoku-py-lib',
    version='1.0.1',
    packages=['tests', 'sudoku'],
    url='',
    license='Apache 2.0',
    author='Vollinger',
    author_email='',
    description='package to generate and solve sudoku puzzles'
)
